@@ -6,8 +6,14 @@
 extern "C" {
 #endif
 
+#include <openssl/err.h>
+#include <openssl/evp.h>
+#include <openssl/rand.h>
+
 #if defined(OQS_USE_OPENSSL)
-void oqs_free_ossl_objects(void);
+void oqs_ossl_destroy(void);
+
+void oqs_thread_stop(void);
 
 const EVP_MD *oqs_sha256(void);
 
@@ -27,9 +33,28 @@ const EVP_MD *oqs_sha3_512(void);
 
 const EVP_CIPHER *oqs_aes_128_ecb(void);
 
+const EVP_CIPHER *oqs_aes_128_ctr(void);
+
 const EVP_CIPHER *oqs_aes_256_ecb(void);
 
 const EVP_CIPHER *oqs_aes_256_ctr(void);
+
+#ifdef OQS_DLOPEN_OPENSSL
+
+#define FUNC(ret, name, args, cargs) ret _oqs_ossl_##name args;
+#define VOID_FUNC FUNC
+#include "ossl_functions.h"
+#undef VOID_FUNC
+#undef FUNC
+
+#define OSSL_FUNC(name) _oqs_ossl_##name
+
+#else
+
+#define OSSL_FUNC(name) name
+
+#endif
+
 #endif
 
 #if defined(__cplusplus)
